class AndroidConnector {
  is_connected = false;

  current_position_requests = [];
  watch_position_requests = [];

  constructor() {
    window.addEventListener("message", this.handleMessageEvent.bind(this));
  }

  handleMessageEvent(event) {
    if (!event.data) return;
    let message = event.data;

    if (!message.from || message.from != "android") return;

    if (message.type == "message") {
      this.handleMessage(message.data);
    } else if (message.type == "geolocation_current_position") {
      this.handleGeolocationCurrentPosition(message.data);
    } else if (message.type == "geolocation_watch_position") {
      this.handleGeolocationWatchPosition(message.data);
    } else {
      console.log(JSON.stringify(event.data));
    }
  }

  handleMessage(message) {
    if (message == "Android App connected") {
      this.is_connected = true;
      console.log("Enabled Android connector");
    }
  }

  handleGeolocationWatchPosition(data) {
    // Check if there was an error with the request
    let request_id = data.request_id;
    if (
      this.watch_position_requests.length - 1 < request_id ||
      this.watch_position_requests[request_id] == null
    )
      return; // No such Request

    if (data.type == "error") {
      this.watch_position_requests[request_id].error_callback(data.error);
    } else if (data.type == "location") {
      let location = data.location;
      if (location.coords.heading == "NaN") {
        location.coords.heading = NaN;
      }
      if (location.coords.altitudeAccuracy == "null") {
        location.coords.altitudeAccuracy = null;
      }
      if (location.coords.heading == "null") {
        location.coords.heading = null;
      }
      if (location.coords.speed == "null") {
        location.coords.speed = null;
      }
      this.watch_position_requests[request_id].success_callback(location);
    }
  }

  handleGeolocationCurrentPosition(data) {
    // Check if there was an error with the request
    let request_id = data.request_id;
    if (
      this.current_position_requests.length - 1 < request_id ||
      this.current_position_requests[request_id] == null
    )
      return; // No such Request
    if (data.type == "error") {
      this.current_position_requests[request_id].error_callback(data.error);
    } else if (data.type == "location") {
      let location = data.location;
      if (location.coords.heading == "NaN") {
        location.coords.heading = NaN;
      }
      if (location.coords.altitudeAccuracy == "null") {
        location.coords.altitudeAccuracy = null;
      }
      if (location.coords.heading == "null") {
        location.coords.heading = null;
      }
      if (location.coords.speed == "null") {
        location.coords.speed = null;
      }
      this.current_position_requests[request_id].success_callback(location);
    }

    // Remove the request
    this.current_position_requests[request_id] = null;
    // Trim all null entries from the end of the array
    this.current_position_requests = this.current_position_requests.slice(
      0,
      this.current_position_requests.length -
      this.current_position_requests
        .reverse()
        .findIndex((element) => element != null)
    );
  }

  getCurrentPosition(success_callback, error_callback, options) {
    if (typeof options == "undefined") {
      console.log("fix options");
      options = {};
    }
    let new_length = this.current_position_requests.push({
      success_callback: success_callback,
      error_callback: error_callback,
      options: options,
    });
    if (options.timeout == Infinity || options.timeout == null) {
      delete options.timeout;
    }
    options.request_id = new_length - 1;
    window.postMessage({
      from: "web",
      type: "geolocation_getcurrentposition",
      data: options,
    });
  }

  watchPosition(success_callback, error_callback, options) {
    let new_length = this.watch_position_requests.push({
      success_callback: success_callback,
      error_callback: error_callback,
      options: options,
    });
    if (options.timeout == Infinity || options.timeout == null) {
      delete options.timeout;
    }
    options.request_id = new_length - 1;
    window.postMessage({
      from: "web",
      type: "geolocation_watchposition",
      data: options,
    });
    return options.request_id;
  }
  clearWatch(id) {
    if (
      this.watch_position_requests.length - 1 < id ||
      this.watch_position_requests[id] == null
    )
      return; // No such Request
    window.postMessage({
      from: "web",
      type: "geolocation_clearwatch",
      data: id,
    });
    // Remove the request
    this.watch_position_requests[id] = null;
    // Trim all null entries from the end of the array
    this.watch_position_requests = this.watch_position_requests.slice(
      0,
      this.watch_position_requests.length -
      this.watch_position_requests
        .reverse()
        .findIndex((element) => element != null)
    );
  }

  /**
   * Function to disable display timeout
   * for example while navigation is on
   */
  toggleDisplayTimeout(enable) {
    window.postMessage({
      from: "web",
      type: "display_timeout_toggle",
      data: enable,
    });
  }
}
