class GpsManager {
  last_location = null;

  #location_possible = null;
  #location_gps_available = null;

  #success_callback = null;
  #error_callback = null;

  #default_options = {
    enableHighAccuracy: true,
    timeout: 10000,
    maximumAge: 60000,
  };
  #watch_callback = (e) => {
    if (e.key == "location") {
      let new_location = e.newValue;
      if (new_location) new_location = JSON.parse(new_location);
      if (new_location.coords.heading == "NaN")
        new_location.coords.heading = NaN;
      this.last_location = new_location;
      this.#success_callback(new_location);
    }
  };

  watchPosition(success_callback, error_callback, options) {
    let location = localStorage.getItem("location");
    if (location) location = JSON.parse(location);
    if (location && Date.now() - location.timestamp < 300000) {
      if (location.coords.heading == "NaN") location.coords.heading = NaN;
      this.#success_callback = success_callback;
      this.#error_callback = error_callback;
      window.addEventListener("storage", this.#watch_callback);
      this.last_location = location;
      success_callback(location);
      return "localstorage";
    } else if (map.androidConnector.is_connected) {
      // Android App is connected. We can use the devices Geolocation
      return map.androidConnector.watchPosition(
        (location) => {
          this.last_location = location;
          success_callback(location);
        },
        error_callback,
        { ...this.#default_options, ...options }
      );
    } else {
      return navigator.geolocation.watchPosition(
        (location) => {
          this.last_location = location;
          success_callback(location);
        },
        error_callback,
        { ...this.#default_options, ...options }
      );
    }
  }

  clearWatch(id) {
    if (id == "localstorage") {
      window.removeEventListener("storage", this.#watch_callback);
    } else if (map.androidConnector.is_connected) {
      map.androidConnector.clearWatch(id);
    } else {
      navigator.geolocation.clearWatch(id);
    }
  }

  getCurrentPosition(success_callback, error_callback, options) {
    // Return cached position if newer than 1s
    if (
      this.last_location &&
      Date.now() - this.last_location.timestamp <= 1000
    ) {
      success_callback(this.last_location);
      return;
    }
    let location = localStorage.getItem("location");
    if (location) location = JSON.parse(location);

    if (location && Date.now() - location.timestamp < 300000) {
      // Valid location from localstorage
      if (location.coords.heading == "NaN") location.coords.heading = NaN;
      this.last_location = location;
      success_callback(location);
    } else if (map.androidConnector.is_connected) {
      // Android App is connected. We can use the devices Geolocation
      map.androidConnector.getCurrentPosition(
        (location) => {
          this.last_location = location;
          success_callback(location);
        },
        error_callback,
        options
      );
    } else {
      navigator.geolocation.getCurrentPosition(
        (location) => {
          this.last_location = location;
          success_callback(location);
        },
        error_callback,
        { ...this.#default_options, ...options }
      );
    }
  }

  async isGeolocationPossible() {
    if (this.#location_possible != null) return this.#location_possible;
    return new Promise((resolve) => {
      this.getCurrentPosition(
        (location) => {
          this.#location_possible = true;
          if (location.coords.accuracy < 150) {
            this.#location_gps_available = true;
          } else {
            this.#location_gps_available = false;
          }
          resolve(true);
        },
        (error) => {
          this.#location_possible = false;
          this.#location_gps_available = false;
          resolve(false);
        },
        {
          timeout: 10000,
          enableHighAccuracy: true,
        }
      );
    });
  }

  async isGPSAvailable() {
    if (this.#location_gps_available != null)
      return this.#location_gps_available;
    return this.isGeolocationPossible().then(() => {
      return this.#location_gps_available;
    });
  }
}
