class SearchModule {
  htmlmodule = document.getElementById("search-addon");
  searchinput = this.htmlmodule.querySelector(".search-input input[name=q]");
  resultscontainer = this.htmlmodule.querySelector(".results");
  searchresultscontainer =
    this.resultscontainer.querySelector(".results-container");
  expandSearchContainer = this.htmlmodule.querySelector(
    ".bounded-actions > .enlarge"
  );
  worldwideSearchContainer = this.htmlmodule.querySelector(
    ".bounded-actions > .worldwide"
  );
  researchContainer = this.htmlmodule.querySelector(
    ".bounded-actions > .research"
  );

  // Callback functions
  moveend_callback = this.updateState.bind(this);
  search_timeout = null;
  stateupdates = true;
  exit_callback = null;

  query = "";
  viewbox = null;
  results = [];
  bounded = null; // Can be enabled|disabled|auto|null  -> indicating whether bounded search was manually enabled/disabled, automatically enabled or automatically disabled
  reverse = null;

  resultHistory = new LocalHistory("results");
  searchHistory = new LocalHistory("suche");

  mapsourcename = "searchmodule"; // Source where all Features of the Routefinder are displayed in
  linesourcename = `${this.mapsourcename}-line`;
  fillsourcename = `${this.mapsourcename}-fill`;
  markers = [];

  reverseCallback = this.handleReverseClick.bind(this);

  constructor() {
    this.addMapLayers();

    // Add exit button functionality
    this.htmlmodule.querySelector(".exit-search").onclick = (e) => {
      e.preventDefault();
      e.stopPropagation();
      this.clearResults();
      this.searchinput.value = "";
      this.query = "";
      this.viewbox = null;
      this.bounded = null;
      this.updateInterface();
      this.updateMap();
      this.updateState();
      if (this.exit_callback) {
        this.exit_callback();
      }
    };

    // Add Waypoint Search function
    let typedsearch = () => {
      if (this.search_timeout) {
        this.search_timeout = clearTimeout(this.search_timeout);
      }

      // Check if input element is now empty
      if (this.searchinput.value.trim().length == 0) {
        this.searchinput.value = "";
        this.clearResults();
        this.query = "";
        this.viewbox = null;
        this.bounded = null;
        this.updateInterface();
        this.updateMap();
        this.updateState();
        return;
      }

      this.search_timeout = setTimeout(() => {
        this.search_timeout = null;
        this.bounded = null;
        this.viewbox = map.map.getBounds();
        this.search()
          .then(() => this.updateInterface())
          .then(() => this.updateMap())
          .then(() => {
            if (["disabled", null].includes(this.bounded)) {
              return this.fitResultsOnMap();
            } else {
              return;
            }
          })
          .then(() => this.updateState())
          .catch((error) => {
            if (!["Query unchanged", "Aborted"].includes(error)) {
              console.trace(error);
            }
          });
      }, 550);
    };
    this.searchinput.onkeyup = typedsearch;

    this.searchinput.addEventListener("focusin", (e) => {
      this.htmlmodule.classList.add("search");
      this.searchresultscontainer.classList.add("search");
      this.searchinput.select();
      if (this.reverse) {
        this.searchinput.value = "";
      }
    });
    this.searchinput.addEventListener("focusout", (e) => {
      if (this.reverse) {
        this.searchinput.value =
          this.reverse.lngLat.lng + " " + this.reverse.lngLat.lat;
      } else if (this.searchinput.value.trim().length == 0) {
        this.htmlmodule.classList.remove("search");
        this.searchresultscontainer.classList.remove("search");
      }
    });

    this.expandSearchContainer.onclick = (e) => {
      e.preventDefault();
      e.stopPropagation();
      let new_zoom = Math.max(map.map.getZoom() - 1, 1);
      map.map.once("zoomend", (e) => {
        this.query = ""; // To Force a new search
        this.viewbox = map.map.getBounds();
        return this.search()
          .then(() => this.updateInterface())
          .then(() => this.updateMap())
          .then(() => this.updateState());
      });
      map.map.easeTo({ zoom: new_zoom }, { user: false });
    };
    this.worldwideSearchContainer.onclick = async (e) => {
      e.preventDefault();
      e.stopPropagation();
      this.bounded = "disabled";
      this.query = "";
      return this.search()
        .then(() => this.updateInterface())
        .then(() => this.updateMap())
        .then(() => this.fitResultsOnMap())
        .then(() => this.updateState());
    };
    this.researchContainer.onclick = async (e) => {
      e.preventDefault();
      e.stopPropagation();
      this.query = "";
      this.viewbox = map.map.getBounds();
      return this.search()
        .then(() => this.updateInterface())
        .then(() => this.updateMap())
        .then(() => this.updateState());
    };
  }

  async handleReverseClick(e) {
    return this.handleReverse(e.lngLat, Math.round(map.map.getZoom()))
      .then(() => this.updateInterface())
      .then(() => this.updateMap())
      .then(() => this.updateState());
  }

  async handleReverse(lngLat, zoom) {
    let base = "/reverse/";

    let params = new URLSearchParams({
      lon: lngLat.lng,
      lat: lngLat.lat,
      zoom: zoom,
      format: "json",
      dedupe: 1,
      extratags: 1,
      addressdetails: 1,
      namedetails: 1,
      polygon_geojson: 1,
    });

    return fetch(base + "?" + params.toString())
      .then((result) => result.json())
      .then((result) => {
        result.rank = 0;
        this.clearResults();
        this.results = [new Result(result, true, "reverse")];

        this.reverse = {
          lngLat: lngLat,
          zoom: zoom,
        };
        this.query = lngLat.lng + " " + lngLat.lat;
        this.viewbox = null;
        this.bounded = null;
        this.searchinput.value = lngLat.lng + " " + lngLat.lat;
        return;
      });
  }

  async enable(
    mapposition = null,
    query,
    viewbox,
    results,
    reverse,
    bounded,
    search = false,
    stateupdates = true,
    exit_callback = null
  ) {
    if (query) {
      this.searchinput.value = query;
      this.htmlmodule.classList.add("search");
    } else {
      this.searchinput.value = "";
    }

    this.stateupdates = stateupdates;
    this.exit_callback = exit_callback;

    if (results) {
      this.clearResults();
      for (const [index, result] of results.entries()) {
        result.rank = index;
        this.results.push(new Result(result, true, result.source));
      }
    }

    if (viewbox) {
      this.viewbox = viewbox;
    }

    this.reverse = reverse;

    if (bounded) {
      this.bounded = bounded;
    }

    let init_promise = Promise.resolve();
    if (query && query.length > 0 && (!results || results.length == 0)) {
      this.htmlmodule.classList.add("active");
      this.viewbox = map.map.getBounds();
      this.bounded = null;
      if (search) {
        // Target is search execution (probably for route finder)
        // searchinput should be focussed
        this.searchinput.focus();
        this.searchinput.selectionStart = this.searchinput.selectionEnd;
        this.searchinput.dispatchEvent(new Event("keyup"));
      } else {
        init_promise = this.search().then(() => this.fitResultsOnMap());
      }
    } else if (this.reverse && this.results.length == 0) {
      init_promise = this.handleReverse(this.reverse.lngLat, this.reverse.zoom);
    }
    return init_promise
      .then(() => this.updateInterface())
      .then(() => this.updateMap())
      .then(() => {
        this.htmlmodule.classList.add("active");
        if (mapposition) {
          let pos = map.map.cameraForBounds(mapposition);
          map.map.easeTo(
            {
              center: pos.center,
              zoom: pos.zoom,
            },
            { user: false }
          );
        }
        this.checkViewbox();
        map.map.off("click", this.reverseCallback);
        map.map.on("click", this.reverseCallback);
        map.map.off("moveend", this.moveend_callback);
        return map.map.on("moveend", this.moveend_callback);
      });
  }

  async search(key_event) {
    let base_url = "/search";
    let query = this.searchinput.value;

    if (query.trim() == this.query.trim()) {
      return Promise.reject("Query unchanged");
    } else {
      this.query = query;
    }

    let params = new URLSearchParams({
      q: query,
      format: "json",
      viewbox: this.viewbox.toArray().toString(),
      dedupe: 1,
      extratags: 1,
      addressdetails: 1,
      namedetails: 1,
      polygon_geojson: 1,
      bounded: this.bounded == "enabled" ? 1 : 0,
    });

    this.htmlmodule.querySelector(".results").classList.add("searching");

    return fetch(base_url + "?" + params.toString())
      .then((response) => response.json())
      .then((response) => {
        if (
          ["auto", null].includes(this.bounded) &&
          response.length > 0 &&
          [
            "retail",
            "supermarket",
            "bank",
            "fuel",
            "clothes",
            "fast_food",
            "playground",
          ].includes(response[0].type)
        ) {
          params.append("bounded", "1");
          this.bounded = "auto";
          return fetch(base_url + "?" + params.toString()).then((response) =>
            response.json()
          );
        } else {
          if (this.bounded == "auto") {
            this.bounded = null;
          }
          return response;
        }
      })
      .then((response) => {
        this.reverse = null;
        this.clearResults(); // Removes all results from the local array / and removes the markers on the way
        for (const [index, result_json] of response.entries()) {
          result_json.rank = index;
          this.results.push(new Result(result_json, true, "search"));
        }

        this.htmlmodule.querySelector(".results").classList.remove("searching");
        return true;
      });
  }

  clearResults() {
    while (true) {
      let result = this.results.pop();
      if (result == undefined) break;
      result.removeMarker();
    }
  }

  /**
   * Shows all results stored in this object
   *
   */
  updateInterface() {
    if (this.searchinput.value.length > 0) {
      this.htmlmodule.classList.add("search");
    } else if (document.activeElement != this.searchinput) {
      this.htmlmodule.classList.remove("search");
    }

    if (["auto", "enabled"].includes(this.bounded)) {
      this.resultscontainer.classList.add("bounded");
    } else {
      this.resultscontainer.classList.remove("bounded");
    }

    if (this.results.length == 0 && this.query.length > 0) {
      this.resultscontainer.classList.add("no-results");
    } else {
      this.resultscontainer.classList.remove("no-results");
    }

    this.searchresultscontainer.textContent = "";

    if (this.results.length > 0) {
      let label = document.createElement("label");
      label.textContent = "Suchergebnisse:";
      this.searchresultscontainer.appendChild(label);
    }
    for (const [index, result] of this.results.entries()) {
      result.updateRank(index);
      this.searchresultscontainer.appendChild(result.dom_element);
      // Add Click listener
      result.dom_element.onclick = (e) => {
        let paddingInline = 250;
        if (window.screen.width < 800) {
          paddingInline = 100;
        }
        map.map.fitBounds(
          [
            [result.data.boundingbox[2], result.data.boundingbox[0]],
            [result.data.boundingbox[3], result.data.boundingbox[1]],
          ],
          {
            duration: 350,
            padding: {
              top: paddingInline,
              bottom: paddingInline,
              right: paddingInline,
              left: paddingInline,
            },
            maxZoom: 16,
          }
        );
      };
      result.dom_element.querySelector(".add-waypoint").onclick = (e) => {
        e.preventDefault();
        e.stopPropagation();
        this.clearResults();
        this.updateInterface();
        this.updateMap();
        let result_data = result.data;
        result_data.rank = 0;
        map.switchModule("route-finding", {
          waypoints: [new Result(result.data, false, result.source)],
        });
      };
    }

    return Promise.resolve();
  }

  updateMap() {
    let layers_and_sources_to_remove = [
      `${this.linesourcename}`,
      `${this.fillsourcename}`,
    ];
    for (const layer_and_source of layers_and_sources_to_remove) {
      let layer_data = map.map.getSource(layer_and_source)._data;
      layer_data.features = [];
      map.map.getSource(layer_and_source).setData(layer_data);
    }
    for (const result of this.results) {
      result.showGeoJson(this.linesourcename, this.fillsourcename);
      // Add Marker on the map
      result.showMarker();
    }
  }

  fitResultsOnMap(all_results = false) {
    if (this.results.length == 0) {
      return;
    }
    let boundingbox = null;
    for (const result of this.results) {
      let result_bbox = turf.bboxPolygon(turf.bbox(result.data.geojson));
      if (boundingbox == null) {
        boundingbox = turf.featureCollection([result_bbox]);
      } else {
        let result_point = turf.point([result.data.lon, result.data.lat]);
        let distance = turf.pointToLineDistance(
          result_point,
          turf.polygonToLineString(turf.bboxPolygon(turf.bbox(boundingbox)))
        );
        if (distance < 100) {
          boundingbox = turf.featureCollection([
            ...boundingbox.features,
            result_bbox,
          ]);
        }
      }
    }
    let bbox = turf.bbox(boundingbox);

    bbox = [
      new maplibregl.LngLat(bbox[0], bbox[1]),
      new maplibregl.LngLat(bbox[2], bbox[3]),
    ];
    const vw = Math.max(
      document.documentElement.clientWidth || 0,
      window.innerWidth || 0
    );
    let paddingInline = 250;
    if (vw < 800) {
      paddingInline = Math.round(vw / 10);
    }
    map.map.fitBounds(bbox, {
      duration: 0,
      padding: {
        top: paddingInline,
        bottom: paddingInline,
        right: paddingInline,
        left: paddingInline,
      },
      maxZoom: 16,
    });
  }

  addMapLayers() {
    // Variables for map features
    let lineSource = {
      type: "geojson",
      data: {
        type: "FeatureCollection",
        features: [],
      },
    };
    let lineLayer = {
      id: `${this.linesourcename}`,
      type: "line",
      source: `${this.linesourcename}`,
      paint: {
        "line-color": "rgb(255,127,0)",
        "line-width": 5,
        "line-opacity": 0.6,
      },
    };

    let fillSource = {
      type: "geojson",
      data: {
        type: "FeatureCollection",
        features: [],
      },
    };
    let fillLayer = {
      id: `${this.fillsourcename}`,
      type: "fill",
      source: `${this.fillsourcename}`,
      paint: {
        "fill-color": "rgb(255,127,0)",
        "fill-outline-color": "rgb(255,127,0)",
        "fill-opacity": 0.3,
      },
    };

    let addLayers = () => {
      map.map.addSource(this.linesourcename, lineSource);
      map.map.addLayer(lineLayer);
      map.map.addSource(this.fillsourcename, fillSource);
      map.map.addLayer(fillLayer);
    };
    if (map.map.isStyleLoaded()) {
      addLayers();
    } else {
      map.map.on("load", addLayers);
    }
  }

  /**
   * Function to tell if the viewbox is close enough to be the same as the current view
   * */
  checkViewbox() {
    if (this.viewbox) {
      let viewboxCamera = map.map.cameraForBounds(this.viewbox);
      if (
        viewboxCamera.center.distanceTo(map.map.getCenter()) > 250 ||
        Math.abs(viewboxCamera.zoom - map.map.getZoom()) >= 0.5
      ) {
        this.htmlmodule
          .querySelector(".bounded-actions")
          .classList.add("research");
        return true;
      } else {
        this.htmlmodule
          .querySelector(".bounded-actions")
          .classList.remove("research");
        return false;
      }
    }
    return false;
  }
  /**
   * Function called when map is loaded and GPS Features become available
   */
  enableGps() {}

  /**
   * Cleanup Code when module is closed. Unregister callbacks etc.
   * Interface will be left as is. Just hidden.
   */
  exit() {
    this.htmlmodule.classList.remove("active");
    map.map.off("click", this.reverseCallback);
    map.map.off("moveend", this.moveend_callback);
  }

  updateState(event) {
    if (
      event &&
      ((event.hasOwnProperty("user") && event.user == false) ||
        (event.hasOwnProperty("0") &&
          ResizeObserverEntry.prototype.isPrototypeOf(event[0])))
    ) {
      return;
    }

    let current_pos = map.map.getBounds();

    this.checkViewbox();

    if (!this.stateupdates) {
      return;
    }

    // Check if current_pos changed enough
    if (
      window.history &&
      window.history.state &&
      window.history.state.module == "search"
    ) {
      let old_pos = new maplibregl.LngLatBounds(
        window.history.state.mapposition._sw,
        window.history.state.mapposition._ne
      );
      let old_pos_camera = map.map.cameraForBounds(old_pos);
      let new_pos_camera = map.map.cameraForBounds(current_pos);
      if (
        old_pos_camera.center.distanceTo(new_pos_camera.center) <= 250 &&
        Math.abs(old_pos_camera.zoom - new_pos_camera.zoom) < 0.5
      ) {
        current_pos = old_pos;
      }
    }

    let url = "";
    if (this.reverse == null) {
      let query =
        this.query.length > 0 ? `${encodeURIComponent(this.query)}/` : "";

      let viewbox = this.viewbox
        ? `${JSON.stringify(this.viewbox.toArray())}/`
        : "";

      url =
        window.location.origin +
        `/map/${query}${viewbox}${JSON.stringify(current_pos.toArray())}`;
    } else {
      url = `/reverse/${this.reverse.lngLat.lng},${this.reverse.lngLat.lat},${
        this.reverse.zoom
      }/${JSON.stringify(current_pos.toArray())}`;
    }

    let results = [];
    for (const result of this.results) {
      results.push(result.data);
    }

    if (["enabled", "disabled"].includes(this.bounded)) {
      url += "?bounded=" + this.bounded;
    }

    let state = {
      module: "search",
      mapposition: current_pos,
      query: this.query,
      viewbox: this.viewbox,
      results: results,
      reverse: this.reverse,
      bounded: this.bounded,
      timestamp: Date.now(),
    };

    if (
      window.history.state == undefined ||
      window.location.href == url ||
      Date.now() - window.history.state.timestamp < 100
    ) {
      window.history.replaceState(state, "", url);
    } else {
      window.history.pushState(state, "", url);
    }
  }
}
