let map = "test";
var mapTmp = null;
$(document).ready(function () {
  $(".inactive").hide();

  // Check if the user called a URL with a location specified
  let bbox, bounded, query, viewbox, vehicle, waypoints; // variables are only predefined if set in URL
  (() => {
    bbox = document.querySelector("meta[name=start-bbox]");
    if (bbox) {
      bbox = bbox.content;
    }
    bounded = document.querySelector("meta[name=start-bounded]");
    if (bounded) {
      bounded = bounded.content;
    }
    viewbox = document.querySelector("meta[name=start-viewbox]");
    if (viewbox) {
      viewbox = viewbox.content;
    }
    query = document.querySelector("meta[name=start-query]");
    if (query) {
      query = query.content;
    }
    reverse_lon = document.querySelector("meta[name=start-reverselon]");
    if (reverse_lon) {
      reverse_lon = parseFloat(reverse_lon.content);
    }
    reverse_lat = document.querySelector("meta[name=start-reverselat]");
    if (reverse_lat) {
      reverse_lat = parseFloat(reverse_lat.content);
    }
    reverse_zoom = document.querySelector("meta[name=start-reversezoom]");
    if (reverse_zoom) {
      reverse_zoom = reverse_zoom.content;
    }
    vehicle = document.querySelector("meta[name=start-vehicle]");
    if (vehicle) {
      vehicle = vehicle.content;
    }
    waypoints = document.querySelector("meta[name=start-waypoints]");
    if (waypoints) {
      waypoints = waypoints.content;
    }
    navigation_active = document.querySelector("meta[name=navigation_active]");
    if (navigation_active) {
      navigation_active = true;
    } else {
      navigation_active = false;
    }
    module = document.querySelector("meta[name=module]");
    if (module) {
      module = module.content;
    }
  })();

  // Parse mapposition
  let center_zooom = null;
  try {
    bbox = JSON.parse(bbox);
    bbox = maplibregl.LngLatBounds.convert(bbox);
  } catch (e) {
    bbox = null;
  }

  if (module) {
    map = new InteractiveMap(bbox, false);
    map.initMap();
    map.map.once("load", () => map.switchModule(module, {}));
  } else if (vehicle && waypoints) {
    // Check if the current url points to a route calculation
    map = new InteractiveMap(bbox, false);
    map.initMap();
    map.map.once("load", () =>
      map.switchModule("route-finding", {
        vehicle: vehicle,
        waypoints: waypoints.split(";"),
        mapposition: bbox,
        navigation_active: navigation_active,
      })
    );
  } else if (bbox || query) {
    map = new InteractiveMap(bbox, false, center_zooom);
    map.initMap();

    // Parse viewbox
    try {
      viewbox = JSON.parse(viewbox);
      viewbox = maplibregl.LngLatBounds.convert(viewbox);
    } catch (e) {
      viewbox = null;
    }

    // Parse reverse
    let reverse = null;
    if (reverse_lon && reverse_lat && reverse_zoom) {
      reverse = {
        lngLat: {
          lng: reverse_lon,
          lat: reverse_lat,
        },
        zoom: reverse_zoom,
      };
    }

    map.map.once("load", () =>
      map.switchModule("search", {
        query: query,
        viewbox: viewbox,
        mapposition: bbox,
        reverse: reverse,
        results: [],
        bounded: bounded,
      })
    );
  } else {
    map = new InteractiveMap();
    map.initMap();

    map.map.once("load", () => {
      // Center the map on the users current position since no position for the map is supplied
      map.gpsManager.getCurrentPosition((position) => {
        // Calculate bbox for current position
        let point = turf.point(
          [position.coords.longitude, position.coords.latitude],
          null,
          { enableHighAccuracy: true }
        );
        let bbox = turf.bbox(
          turf.circle(point, position.coords.accuracy / 1000)
        );
        map.fitBbox(
          [
            [bbox[0], bbox[1]],
            [bbox[2], bbox[3]],
          ],
          17
        );
      }, e => console.error(e), {});
      map.switchModule("search");
    });
  }

  if (
    typeof android != "undefined" &&
    typeof android.pageFinished == "function"
  )
    android.pageFinished();

  window.addEventListener("popstate", (event) => {
    event.preventDefault();
    // Check which module the state is based on
    switch (event.state.module) {
      case "search":
        let mapposition = null;
        if (event.state.mapposition) {
          mapposition = new maplibregl.LngLatBounds(
            event.state.mapposition._sw,
            event.state.mapposition._ne
          );
        }
        let viewbox = null;
        if (event.state.viewbox) {
          viewbox = new maplibregl.LngLatBounds(
            event.state.viewbox._sw,
            event.state.viewbox._ne
          );
        }
        map.switchModule("search", {
          query: event.state.query,
          viewbox: viewbox,
          mapposition: mapposition,
          results: event.state.results,
          reverse: event.state.reverse,
          popstate: true,
        });
        break;
      case "route-finder":
        // Deserialize Waypoints
        let waypoints = [];
        for (const [index, waypoint] of event.state.waypoints.entries()) {
          let route = waypoint.route;
          if (route) {
            route = new Route(route.profile, route.data);
          }
          waypoints.push(
            new Result(waypoint.data, false, waypoint.source, index, route)
          );
        }
        map.switchModule("route-finding", {
          vehicle: vehicle,
          waypoints: waypoints,
          mapposition: event.state.mapposition,
          navigation_active: event.state.navigation_active,
          append: false,
        });
        break;
    }
  });

  // Update data timestamps
  (() => {
    // Tileserver
    fetch("/data/tiles")
      .then((response) => response.json())
      .then((response) => {
        let timestamp = response.timestamp;
        let update_time_string = new Date(timestamp * 1000).toLocaleString();
        document.querySelector(
          "#nav-menu .updates span.data-date-tiles"
        ).textContent = update_time_string;
      });

    // Search
    fetch("/data/search")
      .then((response) => response.json())
      .then((response) => {
        let timestamp = response.timestamp;
        let update_time_string = new Date(timestamp * 1000).toLocaleString();
        document.querySelector(
          "#nav-menu .updates span.data-date-search"
        ).textContent = update_time_string;
      });

    // Routing
    fetch("/data/routing")
      .then((response) => response.json())
      .then((response) => {
        let timestamp = response.timestamp;
        let update_time_string = new Date(timestamp * 1000).toLocaleString();
        document.querySelector(
          "#nav-menu .updates span.data-date-routing"
        ).textContent = update_time_string;
      });

    document.querySelector(
      "#nav-menu .updates span.data-date-timezone"
    ).textContent = Intl.DateTimeFormat().resolvedOptions().timeZone;
  })();
});

// Make navbar links functional
(() => {
  let iframe = document.querySelector("#page iframe");

  document.querySelectorAll("#nav-menu a").forEach((link) => {
    let target = link.href;
    if (target.endsWith("#")) return;
    link.onclick = (e) => {
      if (!target.endsWith("/hilfe") && typeof android == "undefined") return;
      e.preventDefault();
      if (!target.endsWith("/hilfe") && typeof android != "undefined") return;
      iframe.src = target;
      document.querySelector("#page").classList.add("active");
      document.querySelector("#nav-opener").checked = false;
    };
  });
  document.querySelector("#page > div.close-container > button").onclick = (
    e
  ) => {
    iframe.src = "";
    document.querySelector("#page").classList.remove("active");
  };
})();
