<?php

use App\Http\Controllers\DataController;
use App\Http\Controllers\RoutingController;
use App\Http\Controllers\SearchController;
use Illuminate\Support\Facades\Route;
use Illuminate\Http\Request;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
 */

Route::get('/', function () {
    return redirect('map');
});

Route::get('/last-modified', function (Request $request) {
    //sleep(5);
    // List all Files in app, public, resources and routes
    $appIterator = new RecursiveDirectoryIterator(app_path(), RecursiveDirectoryIterator::SKIP_DOTS);
    $publicIterator = new RecursiveDirectoryIterator(public_path(), RecursiveDirectoryIterator::SKIP_DOTS);
    $resourceIterator = new RecursiveDirectoryIterator(resource_path(), RecursiveDirectoryIterator::SKIP_DOTS);
    $routesIterator = new RecursiveDirectoryIterator(base_path('routes'), RecursiveDirectoryIterator::SKIP_DOTS);
    $iterator = new AppendIterator();
    $iterator->append(new RecursiveIteratorIterator($appIterator));
    $iterator->append(new RecursiveIteratorIterator($publicIterator));
    $iterator->append(new RecursiveIteratorIterator($resourceIterator));
    $iterator->append(new RecursiveIteratorIterator($routesIterator));

    $files = [];
    $maxTime = 0;
    foreach ($iterator as $fileInfo) {
        $mtime = filemtime($fileInfo->getPathname());
        if ($maxTime < $mtime) {
            $maxTime = $mtime;
        }
        $files[] = $mtime;
    }

    $response = Response::make("", 200)
        ->header("last-modified", $maxTime * 1000);

    return $response;
});

Route::get('files/list', function (Request $request) {
    $imageIterator = new RecursiveDirectoryIterator(public_path('img'), RecursiveDirectoryIterator::SKIP_DOTS);
    $jsIterator = new RecursiveDirectoryIterator(public_path('js'), RecursiveDirectoryIterator::SKIP_DOTS);
    $cssIterator = new RecursiveDirectoryIterator(public_path('css'), RecursiveDirectoryIterator::SKIP_DOTS);
    $fontsIterator = new RecursiveDirectoryIterator(public_path('fonts'), RecursiveDirectoryIterator::SKIP_DOTS);
    $iterator = new AppendIterator();
    $iterator->append(new RecursiveIteratorIterator($imageIterator));
    $iterator->append(new RecursiveIteratorIterator($jsIterator));
    $iterator->append(new RecursiveIteratorIterator($cssIterator));
    $iterator->append(new RecursiveIteratorIterator($fontsIterator));

    $files = [];
    $delimiter = "MetaGerMaps/public";
    foreach ($iterator as $file) {
        $filePath = $file->getPathname();
        $fileHash = hash_file('sha256', $filePath);
        $pathname = substr($filePath, stripos($filePath, $delimiter) + strlen($delimiter));
        $files[$pathname] = $fileHash;
    }
    $files["/favicon.ico"] = hash_file('sha256', public_path("favicon.ico"));
    $files["/"] = hash_file('sha256', resource_path('views/map.blade.php'));

    return Response::make(json_encode(["offline-data" => $files]), 200)
        ->header("Content-Type", "application/json");
});

Route::group(['prefix' => 'download'], function () {
    Route::get('font', 'DownloadController@downloadFonts');
    Route::get('download-files/{minLon}/{minLat}/{maxLon}/{maxLat}', 'DownloadController@downloadFiles');
    Route::get('download-assets', 'DownloadController@downloadAssets');
    Route::get('list-files/{minLon}/{minLat}/{maxLon}/{maxLat}', 'DownloadController@listFiles');
    Route::get('{minx}/{miny}/{maxx}/{maxy}/{zoomstart}/{zoomend}', 'DownloadController@downloadArea');
});

Route::get('fakegps/{bbox}', function ($bbox) {
    return view('map')->with('css', [mix('css/general.css'), mix('css/mapSearch.css'), mix('css/routing.css')])
        ->with("module", "fakegps")
        ->with("bbox", $bbox);
});
Route::group(['prefix' => 'map'], function () {
    // Map without any URL Parameter
    Route::get('/', function () {
        return view('map')->with('css', [mix('css/general.css'), mix('css/mapSearch.css'), mix('css/routing.css')]);
    });
    // Map with just position defined
    Route::get('{bbox}', function ($bbox) {
        return view('map')->with('css', [mix('css/general.css'), mix('css/mapSearch.css'), mix('css/routing.css')])
            ->with("bbox", $bbox);
    });

    Route::get('{query}/{bbox}', function ($query, $bbox) {
        return view('map')->with('css', [mix('css/general.css'), mix('css/mapSearch.css'), mix('css/routing.css')])
            ->with("bbox", $bbox)
            ->with("query", $query);
    });
    Route::get('{query}/{viewbox}/{bbox}', function ($query, $viewbox, $bbox) {
        return view('map')->with('css', [mix('css/general.css'), mix('css/mapSearch.css'), mix('css/routing.css')])
            ->with("bbox", $bbox)
            ->with("query", $query)
            ->with("viewbox", $viewbox);
    });
});

Route::get("reverse/{reverse_long},{reverse_lat},{reverse_zoom}/{bbox}", function ($reverse_lon, $reverse_lat, $reverse_zoom, $bbox) {
    return view('map')->with('css', [mix('css/general.css'), mix('css/mapSearch.css'), mix('css/routing.css')])
        ->with("bbox", $bbox)
        ->with("reverse_lon", $reverse_lon)
        ->with("reverse_lat", $reverse_lat)
        ->with('reverse_zoom', $reverse_zoom);
});

Route::group(["prefix" => "data"], function () {
    Route::get("tiles", [DataController::class, "tiles"]);
    Route::get("search", [DataController::class, "search"]);
    Route::get("routing", [DataController::class, "routing"]);
});

Route::get("search/search", [SearchController::class, "search"]);
Route::get("lookup", [SearchController::class, "lookup"]);
Route::group(['prefix' => 'reverse'], function () {
    Route::get("/", [SearchController::class, "reverse"]);
    Route::get('{lon}/{lat}', function ($lon, $lat) {
        $link = config("maps.nominatim.host") . "/reverse.php?format=json&lat=$lat&lon=$lon&zoom=18&extratags=1&addressdetails=1&namedetails=1";
        $resContent = file_get_contents($link);
        $response = Response::make($resContent, 200);
        $response->header("Content-Type", "application/json");
        return $response;
    });
});
Route::get('hilfe', function () {
    return view('help');
});
/*
Route::get('about', function () {
// Get File Data date for Openmaptiles
$url = "https://tileserver.metager.de/data/v3/12/2160/1344.pbf";
$headers = get_headers($url, 1);
$lmTiles = $headers["Last-Modified"];

$lmTiles = DateTime::createFromFormat('D, d M Y H:i:s e', $lmTiles);

return view('about')
->with('lmTiles', $lmTiles);
});
 */

/*
Route::group(['prefix' => 'hilfe'], function () {

Route::get('routen-assistent', function () {
return view('routen-assistent')->with('css', [mix('css/staticPages.css')]);
});
});
 */
Route::group(['prefix' => 'route'], function () {
    Route::get("/{vehicle}/{waypoints}/{bbox}", [RoutingController::class, 'map']);
    Route::post("calc", [RoutingController::class, 'calc']);
    Route::get('preview/{vehicle}/{points}', [RoutingController::class, 'routingOverviewGeoJson']);
    Route::get('find/{vehicle}/{points}/{startBearing?}', [RoutingController::class, 'routingGeoJson']);
    Route::get('match/{vehicle}/{points}/{timestamp}/{radiuses}', [RoutingController::class, 'match']);
    Route::get('start/{vehicle}/{points?}', function ($vehicle, $points = "") {
        $waypoints = "[]";
        if ($points !== "") {
            // Let's Convert
            $points = explode(";", $points);
            $waypoints = "[";
            foreach ($points as $index => $value) {
                if ($value === "gps") {
                    $waypoints .= '["gps"],';
                } else {
                    $pos = explode(',', $value);
                    $waypoints .= "[" . $pos[0] . "," . $pos[1] . "],";
                }
            }
            $waypoints = rtrim($waypoints, ",");
            $waypoints .= "]";
        }
        return view('map')->with('css', [mix('css/general.css'), mix('css/mapSearch.css'), mix('css/routing.css')])
            ->with("vars", ["waypoints" => $waypoints, 'vehicle' => $vehicle]);
    });
    Route::get('search/{search}', function ($search) {
        $url = "https://tiles.metager.de/nominatim/search.php?q=" . urlencode($search) . "&limit=5&polygon_geojson=0&format=json&dedupe=1&extratags=1&addressdetails=1&namedetails=1";
        $content = file_get_contents($url);
        $response = Response::make($content, 200);
        $response->header('Content-Type', 'application/json');
        return $response;
    });
    Route::get('{vehicle}/{points}', [RoutingController::class, 'calcRoute']);
});

Route::get('map/{search}/{latMin}/{lonMin}/{latMax}/{lonMax}', function ($search, $latMin, $lonMin, $latMax, $lonMax) {
    return view('map')->with('javascript', "/$search/$latMin/$lonMin/$latMax/$lonMax")->with('search', $search)->with('boundings', 'true')->with('getPosition', 'false')->with('minPos', json_encode([floatval($latMin), floatval($lonMin)]))->with('maxPos', json_encode([floatval($latMax), floatval($lonMax)]))->with('css', [mix('css/mapSearch.css')]);
});

Route::get('{search}/{latMin}/{lonMin}/{latMax}/{lonMax}/{adjustView?}/{limit?}', [SearchController::class, 'boundingBoxSearch']);

Route::get('{search}', [SearchController::class, 'search']);

Route::group(['prefix' => 'metager'], function () {
    Route::get('{search}', [SearchController::class, 'iframeSearch']);
});
